"""
* genfire.utility *

This module contains various useful functions that do not fit into another category


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""


from __future__ import division
import numpy as np
from scipy.interpolate import RegularGridInterpolator
PI = np.pi

try:
    import pyfftw

    #These are just wrappers around various flavors of the FFT
    def rfftn(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.rfftn(arr,overwrite_input=True,threads=threads)
    def irfftn(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.irfftn(arr,overwrite_input=True,threads=threads)
    def rfftn_fftshift(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.rfftn(pyfftw.interfaces.numpy_fft.ifftshift(arr),overwrite_input=True,threads=threads))
    def irfftn_fftshift(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.irfftn(pyfftw.interfaces.numpy_fft.ifftshift(arr),overwrite_input=True,threads=threads))
    def fftn(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.fftn(arr,overwrite_input=True,threads=threads)
    def ifftn(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.ifftn(arr,overwrite_input=True,threads=threads)
    def fftn_fftshift(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.fftn(pyfftw.interfaces.numpy_fft.ifftshift(arr),overwrite_input=True,threads=threads))
    def ifftn_fftshift(arr, threads=6):
        return pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.ifftn(pyfftw.interfaces.numpy_fft.ifftshift(arr),overwrite_input=True,threads=threads))

except ImportError:
    print("No pyFFTW installation found. Installing pyFFTW will significantly improve the speed of GENFIRE. Consult the documentation for more information. Continuing with NumPy FFT...")
    import numpy as np
    def rfftn(arr):
        return np.fft.rfftn(arr)
    def irfftn(arr):
        return np.fft.irfftn(arr)
    def rfftn_fftshift(arr):
        return np.fft.fftshift(np.fft.rfftn(np.fft.ifftshift(arr)))
    def irfftn_fftshift(arr):
        return np.fft.fftshift(np.fft.irfftn(np.fft.ifftshift(arr)))
    def fftn(arr):
        return np.fft.fftn(arr)
    def ifftn(arr):
        return np.fft.ifftn(arr)
    def fftn_fftshift(arr):
        return np.fft.fftshift(np.fft.fftn(np.fft.ifftshift(arr)))
    def ifftn_fftshift(arr):
        return np.fft.fftshift(np.fft.ifftn(np.fft.ifftshift(arr)))


def hermitianSymmetrize(volume):
    """
    * hermitianSymmetrize *

    Enforce Hermitian symmetry to volume

    :param volume: 3D volume to symmetrize
    :return: symmetrized volume
    
    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright (c) 2016. All Rights Reserved.
    """

    startDims = np.shape(volume) # initial dimensions

    # remember the initial dimensions for the end
    dimx = startDims[0]
    dimy = startDims[1]
    dimz = startDims[2]
    flag = False # flag to trigger copying to new odd dimensioned array

    #check if any dimension is odd
    if dimx % 2 == 0:
        dimx += 1
        flag = True

    if dimy % 2 == 0:
        dimy += 1
        flag = True

    if dimz % 2 == 0:
        dimz += 1
        flag = True

    if flag: # if any dimensions are even, create a new with all odd dimensions and copy volume
        newInput = np.zeros((dimx,dimy,dimz), dtype=complex) #new array
        newInput[:startDims[0], :startDims[1], :startDims[2]] = volume # copy values
        numberOfValues = (newInput != 0).astype(float) #track number of values for averaging
        newInput = newInput + np.conj(newInput[::-1, ::-1, ::-1]) # combine Hermitian symmetry mates
        numberOfValues = numberOfValues + numberOfValues[::-1, ::-1, ::-1] # track number of points included in each sum

        newInput[numberOfValues != 0] /= numberOfValues[numberOfValues != 0] # take average where two values existed
        newInput[np.isnan(newInput)] = 0 # Shouldn't be any nan, but just to be safe

        return newInput[:startDims[0], :startDims[1], :startDims[2]] # return original dimensions


    else: # otherwise, save yourself the trouble of copying the matrix over. See previous comments for line-by-line
        numberOfValues = (volume != 0).astype(float)
        volume = volume + np.conjugate(volume[::-1, ::-1, ::-1])
        numberOfValues = numberOfValues + numberOfValues[::-1, ::-1, ::-1]
        volume[numberOfValues != 0] /= numberOfValues[numberOfValues != 0]
        volume[np.isnan(volume)] = 0
        return volume


def smooth3D(object,resolutionCutoff):
    """
    * smooth3D *

    Low pass filter a 3D volume

    :param object: 3D volume
    :param resolutionCutoff: Fraction of Nyquist frequency to use as sigma for the Gaussian filter
    :return: smoothed volume

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright (c) 2016. All Rights Reserved.
    """
    dims = np.shape(object)
    if dims[2] == 1:
        raise Exception('This is not a 3D object, use smooth2D instead.')
    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)
    Zcenter = round(dims[2]//2)

    # These  next few lines are easier to read, but in the actual code I just directly
    # place the np.arange terms in to avoid creating unnecessary variables
    # Xvec = np.arange(0,dims[0])-Xcenter
    # Yvec = np.arange(0,dims[1])-Ycenter
    # Zvec = np.arange(0,dims[2])-Zcenter
    # xx, yy, zz = np.meshgrid(Xvec, Yvec, Zvec)

    xx, yy, zz = np.meshgrid(np.arange(0,dims[0])-Xcenter, np.arange(0,dims[1])-Ycenter, np.arange(0,dims[2])-Zcenter)
    sigma  = dims[0]/2*resolutionCutoff

    #construct the filter and normalize
    K_filter = np.exp(-(((xx)**2 + (yy)**2 + (zz) **2)/(2*sigma*sigma)))
    K_filter /= np.max(np.max(np.max(abs(K_filter))))

    # take FFT and multiply by filter
    # kbinned = pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.fftn(object,overwrite_input=True)) * K_filter
    kbinned = fftn_fftshift(object) * K_filter

    # Assuming the input is real, the output will be approximately real, but will have some
    # tiny imaginary part due to rounding errors. This function would work for smoothing a
    # complex object, but this return statement would need to be modified
    # return np.real(pyfftw.interfaces.numpy_fft.ifftn(pyfftw.interfaces.numpy_fft.ifftshift(kbinned)))
    return np.real(fftn_fftshift(kbinned))



def smooth2D(object,resolutionCutoff):
    """
    * smooth2D *

    Low pass filter a 2D image

    :param object: 2D image
    :param resolutionCutoff: Fraction of Nyquist frequency to use as sigma for the Gaussian filter
    :return: smoothed image

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright (c) 2016. All Rights Reserved.
    """
    dims = np.shape(object)
    if len(dims)>2:
        raise Exception('This is a 3D object, use smooth3D instead.')
    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)

    # These  next few lines are easier to read, but in the actual code I just directly
    # place the np.arange terms in to avoid creating unnecessary variables
    # Xvec = np.arange(0,dims[0])-Xcenter
    # Yvec = np.arange(0,dims[1])-Ycenter
    # xx, yy = np.meshgrid(Xvec, Yvec)

    xx, yy = np.meshgrid(np.arange(0,dims[0])-Xcenter, np.arange(0,dims[1])-Ycenter)
    sigma  = dims[0]/2*resolutionCutoff

    #construct the filter and normalize
    K_filter = np.exp(-(((xx)**2 + (yy)**2)/(2*sigma*sigma)))
    K_filter /= np.max(np.max(np.max(abs(K_filter))))

    # take FFT and multiply by filter
    # kbinned =  pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.fftn(object,overwrite_input=True)) * K_filter
    kbinned = fftn_fftshift(object) * K_filter
    # Assuming the input is real, the output will be approximately real, but will have some
    # tiny imaginary part due to rounding errors. This function would work for smoothing a
    # complex object, but this return statement would need to be modified
    # return np.real(pyfftw.interfaces.numpy_fft.ifftn(pyfftw.interfaces.numpy_fft.ifftshift(kbinned)))
    return np.real(ifftn_fftshift(kbinned))


def calculateProjection_interp(modelK, phi, theta, psi):
    """
    * calculateProjection_interp *

    Calculate a projection of a 3D volume from it's oversampled Fourier transform by interpolating
    the central slice at the orientation determined by Euler angles (phi, theta, psi)

    :param modelK: numpy array holding the oversampled FFT of the model
    :param phi: euler angle 1
    :param theta: euler angle 2
    :param psi: euler angle 3
    :return: projection
    """

    # get dimensions
    dims = np.shape(modelK)
    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)
    Zcenter = round(dims[2]//2)

    # convert angles to radians
    phi *= PI/180
    theta *= PI/180
    psi *= PI/180

    # construct rotation matrix
    R = np.array([[np.cos(psi)*np.cos(theta)*np.cos(phi)-np.sin(psi)*np.sin(phi) , np.cos(psi)*np.cos(theta)*np.sin(phi)+np.sin(psi)*np.cos(phi)   ,    -np.cos(psi)*np.sin(theta)],
    [-np.sin(psi)*np.cos(theta)*np.cos(phi)-np.cos(psi)*np.sin(phi), -np.sin(psi)*np.cos(theta)*np.sin(phi)+np.cos(psi)*np.cos(phi) ,   np.sin(psi)*np.sin(theta) ],
    [np.sin(theta)*np.cos(phi)                               , np.sin(theta)*np.sin(phi)                                ,              np.cos(theta)]])

    R = R.T

    kx = np.arange(0, dims[0])-Xcenter
    ky = np.arange(0, dims[1])-Ycenter
    kz = np.arange(0, dims[2])-Zcenter

    # construct interpolator function that does the actual computation
    interpolator = RegularGridInterpolator((kx, ky, kz), modelK, bounds_error=False, fill_value=0)

    # build coordinates of the slice we want to calculate
    kx_slice, ky_slice, kz_slice = np.meshgrid((np.arange(0, dims[0])-Xcenter), (np.arange(0, dims[1])-Ycenter), 0)

    # rotate coordinates
    rotKCoords = np.zeros([3, np.size(kx_slice)])
    rotKCoords[0, :] = np.reshape(kx_slice, [1, np.size(kx_slice)])
    rotKCoords[1, :] = np.reshape(ky_slice, [1, np.size(ky_slice)])
    rotKCoords[2, :] = np.reshape(kz_slice, [1, np.size(kz_slice)])
    rotKCoords = np.dot(R, rotKCoords)

    # calculate projection
    projection = interpolator(rotKCoords.T)
    projection = np.reshape(projection, [dims[0], dims[1]], order='F')

    return np.real(ifftn_fftshift(projection))

def getProjectionInterpolator(modelK):
    """
    * generateKspaceIndices *

    Maps the radial coordinate indices in the matrix obj

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.


    :param obj: Matrix of size to be mapped
    :return: 3D indices for each voxel in the volume
    """

    # get dimensions
    dims = np.shape(modelK)
    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)
    Zcenter = round(dims[2]//2)

    kx = np.arange(0, dims[0])-Xcenter
    ky = np.arange(0, dims[1])-Ycenter
    kz = np.arange(0, dims[2])-Zcenter

    # construct interpolator function that does the actual computation
    return RegularGridInterpolator((kx, ky, kz), modelK, bounds_error=False, fill_value=0)

def calculateProjection_interp_fromInterpolator(interpolator, phi, theta, psi, dims):
    """
    * calculateProjection_interp_fromInterpolator *

    Calculate a projection from precomputed interpolator object

    :param interpolator: RegularGridInterpolator object from scipy.
    :param phi: euler angle 1
    :param theta: euler angle 2
    :param psi: euler angle 3
    :param dims: dimensions of the object

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """

    # convert to radians
    phi *= PI/180
    theta *= PI/180
    psi *= PI/180

    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)
    Zcenter = round(dims[2]//2)

    # construct rotation matrix
    R = np.array([[np.cos(psi)*np.cos(theta)*np.cos(phi)-np.sin(psi)*np.sin(phi) , np.cos(psi)*np.cos(theta)*np.sin(phi)+np.sin(psi)*np.cos(phi)   ,    -np.cos(psi)*np.sin(theta)],
    [-np.sin(psi)*np.cos(theta)*np.cos(phi)-np.cos(psi)*np.sin(phi), -np.sin(psi)*np.cos(theta)*np.sin(phi)+np.cos(psi)*np.cos(phi) ,   np.sin(psi)*np.sin(theta) ],
    [np.sin(theta)*np.cos(phi)                               , np.sin(theta)*np.sin(phi)                                ,              np.cos(theta)]])

    R = R.T

    # build coordinates of the slice we want to calculate
    kx_slice, ky_slice, kz_slice = np.meshgrid((np.arange(0, dims[0])-Xcenter), (np.arange(0, dims[1])-Ycenter), 0)

    # rotate coordinates
    rotKCoords = np.zeros([3, np.size(kx_slice)])
    rotKCoords[0, :] = np.reshape(kx_slice, [1, np.size(kx_slice)])
    rotKCoords[1, :] = np.reshape(ky_slice, [1, np.size(ky_slice)])
    rotKCoords[2, :] = np.reshape(kz_slice, [1, np.size(kz_slice)])
    rotKCoords = np.dot(R, rotKCoords)

    # calculate projection
    projection = interpolator(rotKCoords.T)
    projection = np.reshape(projection, [dims[0], dims[1]], order='F')

    return np.real(ifftn_fftshift(projection))


def calculateProjection_DFT(model, phi, theta, psi, out_dimx, out_dimy):
    """
    * calculateProjection_interp *

    Calculate a projection of a 3D volume using the Discrete Fourier Transform (DFT)

    :param model: numpy array holding the model
    :param phi: euler angle 1
    :param theta: euler angle 2
    :param psi: euler angle 3
    :param out_dimx: output projection size in x
    :param out_dimy: output projection size in y
    :return: projection
    """

    # get dimensions
    dims = np.shape(model)
    Xcenter = round(dims[0]//2)
    Ycenter = round(dims[1]//2)
    Zcenter = round(dims[2]//2)

    KXcenter = round(out_dimx//2)
    KYcenter = round(out_dimy//2)

    # convert to radians
    phi *= PI/180
    theta *= PI/180
    psi *= PI/180

    R = np.array([[np.cos(psi)*np.cos(theta)*np.cos(phi)-np.sin(psi)*np.sin(phi) , np.cos(psi)*np.cos(theta)*np.sin(phi)+np.sin(psi)*np.cos(phi)   ,    -np.cos(psi)*np.sin(theta)],
    [-np.sin(psi)*np.cos(theta)*np.cos(phi)-np.cos(psi)*np.sin(phi), -np.sin(psi)*np.cos(theta)*np.sin(phi)+np.cos(psi)*np.cos(phi) ,   np.sin(psi)*np.sin(theta) ],
    [np.sin(theta)*np.cos(phi)                               , np.sin(theta)*np.sin(phi)                                ,              np.cos(theta)]])

    R = R.T

    # build coordinates of 3D FFT

    X = np.arange(0, dims[0])-Xcenter
    Y = np.arange(0, dims[1])-Ycenter
    Z = np.arange(0, dims[2])-Zcenter
    X, Y, Z = np.meshgrid(X,Y,Z)
    Y = Y.astype(float)
    X = X.astype(float)
    Z = Z.astype(float)

    KX = np.arange(0, out_dimx)-KXcenter
    KY = np.arange(0, out_dimy)-KYcenter
    KY, KX, KZ = np.meshgrid(KY, KX, 0)
    KY = KY.astype(float)
    KX = KX.astype(float)
    KZ = KZ.astype(float)

    # build coordinates of the slice we want to calculate
    # kx_slice, ky_slice, kz_slice = np.meshgrid((np.arange(0, dims[0])-Xcenter), (np.arange(0, dims[1])-Ycenter), 0)

    # rotate coordinates
    rotKCoords = np.zeros([3, np.size(KX)])
    rotKCoords[0, :] = np.reshape(KX, [1, np.size(KX)])
    rotKCoords[1, :] = np.reshape(KY, [1, np.size(KY)])
    rotKCoords[2, :] = np.reshape(KZ, [1, np.size(KZ)])
    rotKCoords = np.dot(R, rotKCoords)

    rotKX = rotKCoords[0, :]
    rotKY = rotKCoords[1, :]
    rotKZ = rotKCoords[2, :]

    projection = np.zeros(out_dimx*out_dimy,dtype=complex)
    ind = np.where(model!=0) # only consider nonzero indices for speed
    model = model[ind]
    X = X[ind]
    Y = Y[ind]
    Z = Z[ind]
    for i in range(np.size(projection)): # compute DFT
        projection[i] = np.sum( model * np.exp( -2*PI*1j * \
                          ( rotKX[i]*X/out_dimx + rotKY[i]*Y/out_dimy + rotKZ[i]*Z/out_dimx ) ) )
    projection = np.reshape(projection, (out_dimx, out_dimy), order='F')

    # return np.real(pyfftw.interfaces.numpy_fft.fftshift(pyfftw.interfaces.numpy_fft.ifftn(pyfftw.interfaces.numpy_fft.ifftshift(projection))))
    return np.real(ifftn_fftshift(projection))


def generateKspaceIndices(obj):
    """
    * generateKspaceIndices *

    Maps the radial coordinate indices in the matrix obj

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.


    :param obj: Matrix of size to be mapped
    :return: 3D indices for each voxel in the volume
    """

    dims = np.shape(obj)
    if len(dims) < 3:
        dims = dims + (0,)

    if dims[0] % 2 == 0:
        ncK0 = dims[0]/2
        vec0 = np.arange(-ncK0, ncK0, 1)/ncK0
    elif dims[0] == 1:
        vec0 = 0
        ncK0 = 1

    else:
        ncK0 = ((dims[0]+1)/2)-1
        vec0 = np.arange(-ncK0, ncK0+1)/ncK0


    if dims[1] % 2 == 0:
        ncK1 = dims[1]/2
        vec1 = np.arange(-ncK1, ncK1, 1)/ncK1
    elif dims[1] == 1:
        vec1 = 0
        ncK1 = 1

    else:
        ncK1 = ((dims[1]+1)/2)-1
        vec1 = np.arange(-ncK1, ncK1+1)/ncK1


    if dims[2] % 2 == 0:
        ncK2 = dims[2]/2
        vec2 = np.arange(-ncK2, ncK2, 1)/ncK2
    elif dims[2] == 1:
        vec2 = 0
        ncK2 = 1

    else:
        ncK2 = ((dims[2]+1)/2)-1
        vec2 = np.arange(-ncK2, ncK2+1)/ncK2

    kx, ky, kz = np.meshgrid(vec1,vec0,vec2)
    Kindices = np.sqrt(kx**2 + ky**2 + kz**2)
    return Kindices

def pointToPlaneDistance(points, norm_vec):
    """
    * pointToPlaneDistance *

    compute distance from point to a plane with normal vector norm_vec passing through origin

    :param points: num_points x 3 array of (x,y,z) points
    :param norm_vec: (a,b,c) normal vector defining the plane ax + by + cz = 0
    :return: numpy array containing distance to plane

    Author: Yongsoo Yang
    Transcribed from MATLAB by Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """
    from numpy.linalg import norm
    norm_vec = norm_vec / norm(norm_vec)
    if np.ndim(points)>1:
        distances = np.abs(np.sum(points*norm_vec,axis=1))
        return distances
    return np.abs(np.dot(points,norm_vec))

def pointToPlaneClosest(points, norm_vec, distances):
    """
    * pointToPlaneClosest *

    compute the closest point in the plane defined by normal vector norm_vec to each
    point in points

    :param points: num_points x 3 array of (x,y,z) points
    :param norm_vec: (a,b,c) normal vector defining the plane ax + by + cz = 0
    :param distances: magnitude of distance to plane (see pointToPlaneDistance)
    :return:  numpy array containing the coordinates of the closest points

    Author: Yongsoo Yang
    Transcribed from MATLAB codes by Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """
    if np.ndim(points)>1: # use NumPy broadcasting for speed
        closest_points = points + ( (distances - np.sum(points * norm_vec)) * np.reshape(norm_vec,(3,1)) / np.dot(norm_vec,norm_vec) ).T
        return closest_points
    return points + (distances- np.dot(points, norm_vec)) * norm_vec




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'GENFIRE_MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GENFIRE_MainWindow(object):
    def setupUi(self, GENFIRE_MainWindow):
        GENFIRE_MainWindow.setObjectName("GENFIRE_MainWindow")
        GENFIRE_MainWindow.setEnabled(True)
        GENFIRE_MainWindow.resize(807, 876)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GENFIRE_MainWindow.sizePolicy().hasHeightForWidth())
        GENFIRE_MainWindow.setSizePolicy(sizePolicy)
        self.centralWidget = QtWidgets.QWidget(GENFIRE_MainWindow)
        self.centralWidget.setObjectName("centralWidget")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.centralWidget)
        self.gridLayout_4.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_4.setSpacing(6)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.scrollArea = QtWidgets.QScrollArea(self.centralWidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 799, 971))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_4.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_4.setSpacing(6)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout_3.setSpacing(6)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.nameWidget = QtWidgets.QWidget(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nameWidget.sizePolicy().hasHeightForWidth())
        self.nameWidget.setSizePolicy(sizePolicy)
        self.nameWidget.setMinimumSize(QtCore.QSize(512, 93))
        self.nameWidget.setStyleSheet("background-image: url(:/GENFIRE.png)")
        self.nameWidget.setObjectName("nameWidget")
        self.horizontalLayout_3.addWidget(self.nameWidget)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_3.setSpacing(6)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.lineEdit_pj = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.lineEdit_pj.setObjectName("lineEdit_pj")
        self.gridLayout.addWidget(self.lineEdit_pj, 0, 2, 1, 1)
        self.line_4 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.gridLayout.addWidget(self.line_4, 10, 0, 1, 3)
        self.lineEdit_io = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.lineEdit_io.setObjectName("lineEdit_io")
        self.gridLayout.addWidget(self.lineEdit_io, 9, 2, 1, 1)
        self.line_2 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout.addWidget(self.line_2, 5, 0, 1, 3)
        self.label_4 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 12, 0, 1, 1)
        self.line = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 1, 0, 1, 3)
        self.line_3 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout.addWidget(self.line_3, 7, 0, 1, 3)
        self.label_5 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 13, 2, 1, 1)
        self.lineEdit_results = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.lineEdit_results.setObjectName("lineEdit_results")
        self.gridLayout.addWidget(self.lineEdit_results, 12, 2, 1, 1)
        self.lineEdit_angle = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.lineEdit_angle.setObjectName("lineEdit_angle")
        self.gridLayout.addWidget(self.lineEdit_angle, 6, 2, 1, 1)
        self.checkBox_provide_io = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_provide_io.setObjectName("checkBox_provide_io")
        self.gridLayout.addWidget(self.checkBox_provide_io, 8, 0, 1, 1)
        self.btn_projections = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        self.btn_projections.setObjectName("btn_projections")
        self.gridLayout.addWidget(self.btn_projections, 0, 0, 1, 1)
        self.btn_angles = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        self.btn_angles.setObjectName("btn_angles")
        self.gridLayout.addWidget(self.btn_angles, 6, 0, 1, 1)
        self.btn_support = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        self.btn_support.setObjectName("btn_support")
        self.gridLayout.addWidget(self.btn_support, 3, 0, 1, 1)
        self.lineEdit_support = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.lineEdit_support.setObjectName("lineEdit_support")
        self.gridLayout.addWidget(self.lineEdit_support, 3, 2, 1, 1)
        self.btn_io = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        self.btn_io.setObjectName("btn_io")
        self.gridLayout.addWidget(self.btn_io, 9, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout.addLayout(self.horizontalLayout, 13, 0, 1, 1)
        self.checkBox_default_support = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_default_support.setObjectName("checkBox_default_support")
        self.gridLayout.addWidget(self.checkBox_default_support, 2, 0, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_2)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_on = QtWidgets.QRadioButton(self.groupBox)
        self.radioButton_on.setObjectName("radioButton_on")
        self.verticalLayout.addWidget(self.radioButton_on)
        self.radioButton_off = QtWidgets.QRadioButton(self.groupBox)
        self.radioButton_off.setObjectName("radioButton_off")
        self.verticalLayout.addWidget(self.radioButton_off)
        self.radioButton_extension = QtWidgets.QRadioButton(self.groupBox)
        self.radioButton_extension.setObjectName("radioButton_extension")
        self.verticalLayout.addWidget(self.radioButton_extension)
        self.horizontalLayout_2.addWidget(self.groupBox)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 4, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 4, 0, 1, 1)
        self.lineEdit_numIterations = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_numIterations.sizePolicy().hasHeightForWidth())
        self.lineEdit_numIterations.setSizePolicy(sizePolicy)
        self.lineEdit_numIterations.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_numIterations.setObjectName("lineEdit_numIterations")
        self.gridLayout_2.addWidget(self.lineEdit_numIterations, 0, 1, 1, 1)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_3.setSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_2.addLayout(self.gridLayout_3, 0, 5, 7, 1)
        self.lineEdit_oversamplingRatio = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_oversamplingRatio.sizePolicy().hasHeightForWidth())
        self.lineEdit_oversamplingRatio.setSizePolicy(sizePolicy)
        self.lineEdit_oversamplingRatio.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_oversamplingRatio.setObjectName("lineEdit_oversamplingRatio")
        self.gridLayout_2.addWidget(self.lineEdit_oversamplingRatio, 4, 1, 1, 1)
        self.lineEdit_interpolationCutoffDistance = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_interpolationCutoffDistance.sizePolicy().hasHeightForWidth())
        self.lineEdit_interpolationCutoffDistance.setSizePolicy(sizePolicy)
        self.lineEdit_interpolationCutoffDistance.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_interpolationCutoffDistance.setObjectName("lineEdit_interpolationCutoffDistance")
        self.gridLayout_2.addWidget(self.lineEdit_interpolationCutoffDistance, 6, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 6, 0, 1, 1)
        self.line_5 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.gridLayout_2.addWidget(self.line_5, 0, 3, 1, 1)
        self.label = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label.setWordWrap(False)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 4, 4, 1, 1)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_5.setSpacing(6)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_6 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_5.addWidget(self.label_6)
        self.checkBox_positivity_constraint = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_positivity_constraint.setObjectName("checkBox_positivity_constraint")
        self.verticalLayout_5.addWidget(self.checkBox_positivity_constraint)
        self.checkBox_support_constraint = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_support_constraint.setObjectName("checkBox_support_constraint")
        self.verticalLayout_5.addWidget(self.checkBox_support_constraint)
        self.gridLayout_2.addLayout(self.verticalLayout_5, 1, 3, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_2)
        self.groupBox_2.setTitle("")
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_7.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_7.setSpacing(6)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.radioButton_FFT = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_FFT.setObjectName("radioButton_FFT")
        self.verticalLayout_7.addWidget(self.radioButton_FFT)
        self.radioButton_DFT = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_DFT.setObjectName("radioButton_DFT")
        self.verticalLayout_7.addWidget(self.radioButton_DFT)
        self.gridLayout_2.addWidget(self.groupBox_2, 1, 1, 1, 1)
        self.line_6 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.gridLayout_2.addWidget(self.line_6, 2, 3, 1, 1)
        self.btn_displayResults = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        self.btn_displayResults.setObjectName("btn_displayResults")
        self.gridLayout_2.addWidget(self.btn_displayResults, 6, 3, 1, 1)
        self.checkBox_rfree = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_rfree.setObjectName("checkBox_rfree")
        self.gridLayout_2.addWidget(self.checkBox_rfree, 4, 3, 1, 1)
        self.checkBox_resCircle = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_resCircle.setChecked(True)
        self.checkBox_resCircle.setObjectName("checkBox_resCircle")
        self.gridLayout_2.addWidget(self.checkBox_resCircle, 2, 0, 1, 2)
        self.checkBox_multiGridding = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_2)
        self.checkBox_multiGridding.setChecked(True)
        self.checkBox_multiGridding.setObjectName("checkBox_multiGridding")
        self.gridLayout_2.addWidget(self.checkBox_multiGridding, 3, 0, 1, 2)
        self.horizontalLayout_2.addLayout(self.gridLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout_4.setSpacing(6)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btn_reconstruct = QtWidgets.QPushButton(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_reconstruct.sizePolicy().hasHeightForWidth())
        self.btn_reconstruct.setSizePolicy(sizePolicy)
        self.btn_reconstruct.setMinimumSize(QtCore.QSize(775, 50))
        self.btn_reconstruct.setObjectName("btn_reconstruct")
        self.horizontalLayout_4.addWidget(self.btn_reconstruct)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.log = QtWidgets.QTextEdit(self.scrollAreaWidgetContents_2)
        self.log.setMinimumSize(QtCore.QSize(100, 0))
        self.log.setObjectName("log")
        self.verticalLayout_2.addWidget(self.log)
        self.verticalLayout_4.addLayout(self.verticalLayout_2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)
        self.gridLayout_4.addWidget(self.scrollArea, 0, 0, 1, 1)
        GENFIRE_MainWindow.setCentralWidget(self.centralWidget)
        self.mainToolBar = QtWidgets.QToolBar(GENFIRE_MainWindow)
        self.mainToolBar.setObjectName("mainToolBar")
        GENFIRE_MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.mainToolBar)
        self.statusBar = QtWidgets.QStatusBar(GENFIRE_MainWindow)
        self.statusBar.setObjectName("statusBar")
        GENFIRE_MainWindow.setStatusBar(self.statusBar)
        self.menuBar = QtWidgets.QMenuBar(GENFIRE_MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 807, 22))
        self.menuBar.setObjectName("menuBar")
        self.menuTools = QtWidgets.QMenu(self.menuBar)
        self.menuTools.setObjectName("menuTools")
        self.menuView_Volume = QtWidgets.QMenu(self.menuBar)
        self.menuView_Volume.setObjectName("menuView_Volume")
        GENFIRE_MainWindow.setMenuBar(self.menuBar)
        self.action_Create_Support = QtWidgets.QAction(GENFIRE_MainWindow)
        self.action_Create_Support.setObjectName("action_Create_Support")
        self.action_Volume_Slicer = QtWidgets.QAction(GENFIRE_MainWindow)
        self.action_Volume_Slicer.setObjectName("action_Volume_Slicer")
        self.action_2 = QtWidgets.QAction(GENFIRE_MainWindow)
        self.action_2.setObjectName("action_2")
        self.menuTools.addAction(self.action_Create_Support)
        self.menuView_Volume.addAction(self.action_Volume_Slicer)
        self.menuBar.addAction(self.menuTools.menuAction())
        self.menuBar.addAction(self.menuView_Volume.menuAction())

        self.retranslateUi(GENFIRE_MainWindow)
        QtCore.QMetaObject.connectSlotsByName(GENFIRE_MainWindow)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_pj, self.lineEdit_support)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_support, self.lineEdit_angle)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_angle, self.lineEdit_io)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_io, self.lineEdit_numIterations)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_numIterations, self.lineEdit_oversamplingRatio)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_oversamplingRatio, self.lineEdit_interpolationCutoffDistance)
        GENFIRE_MainWindow.setTabOrder(self.lineEdit_interpolationCutoffDistance, self.radioButton_on)
        GENFIRE_MainWindow.setTabOrder(self.radioButton_on, self.radioButton_off)
        GENFIRE_MainWindow.setTabOrder(self.radioButton_off, self.radioButton_extension)
        GENFIRE_MainWindow.setTabOrder(self.radioButton_extension, self.btn_angles)
        GENFIRE_MainWindow.setTabOrder(self.btn_angles, self.btn_io)
        GENFIRE_MainWindow.setTabOrder(self.btn_io, self.btn_projections)
        GENFIRE_MainWindow.setTabOrder(self.btn_projections, self.btn_support)

    def retranslateUi(self, GENFIRE_MainWindow):
        _translate = QtCore.QCoreApplication.translate
        GENFIRE_MainWindow.setWindowTitle(_translate("GENFIRE_MainWindow", "GENFIRE"))
        self.label_4.setText(_translate("GENFIRE_MainWindow", "Results Filename"))
        self.label_5.setText(_translate("GENFIRE_MainWindow", "*.mrc, *.npy, or *.mat"))
        self.checkBox_provide_io.setText(_translate("GENFIRE_MainWindow", "Provide initial object"))
        self.btn_projections.setText(_translate("GENFIRE_MainWindow", "Select Projection File"))
        self.btn_angles.setText(_translate("GENFIRE_MainWindow", "Select Angle File"))
        self.btn_support.setText(_translate("GENFIRE_MainWindow", "Select Support File"))
        self.btn_io.setText(_translate("GENFIRE_MainWindow", "Select Inital Object File"))
        self.checkBox_default_support.setText(_translate("GENFIRE_MainWindow", "Use default support"))
        self.groupBox.setTitle(_translate("GENFIRE_MainWindow", "Resolution Extension/Suppression"))
        self.radioButton_on.setText(_translate("GENFIRE_MainWindow", "On"))
        self.radioButton_off.setText(_translate("GENFIRE_MainWindow", "Off"))
        self.radioButton_extension.setText(_translate("GENFIRE_MainWindow", "Extension Only"))
        self.label_2.setText(_translate("GENFIRE_MainWindow", "Oversampling Ratio"))
        self.label_3.setText(_translate("GENFIRE_MainWindow", "Interpolation Cutoff Distance (pixels)"))
        self.label.setText(_translate("GENFIRE_MainWindow", "Number of Iterations"))
        self.label_6.setText(_translate("GENFIRE_MainWindow", "Constraints"))
        self.checkBox_positivity_constraint.setText(_translate("GENFIRE_MainWindow", "Positivity"))
        self.checkBox_support_constraint.setText(_translate("GENFIRE_MainWindow", "Support"))
        self.label_7.setText(_translate("GENFIRE_MainWindow", "Gridding Method"))
        self.radioButton_FFT.setText(_translate("GENFIRE_MainWindow", "FFT"))
        self.radioButton_DFT.setText(_translate("GENFIRE_MainWindow", "DFT"))
        self.btn_displayResults.setText(_translate("GENFIRE_MainWindow", "Summarize Results"))
        self.checkBox_rfree.setText(_translate("GENFIRE_MainWindow", "Calculate Rfree"))
        self.checkBox_resCircle.setText(_translate("GENFIRE_MainWindow", "Enforce Resolution Circle"))
        self.checkBox_multiGridding.setText(_translate("GENFIRE_MainWindow", "Permit Multiple Grid-Point Matches"))
        self.btn_reconstruct.setText(_translate("GENFIRE_MainWindow", "Launch Reconstruction"))
        self.menuTools.setTitle(_translate("GENFIRE_MainWindow", "Projection Calculator"))
        self.menuView_Volume.setTitle(_translate("GENFIRE_MainWindow", "View Volume"))
        self.action_Create_Support.setText(_translate("GENFIRE_MainWindow", "Launch Projection Calculator"))
        self.action_Volume_Slicer.setText(_translate("GENFIRE_MainWindow", "Launch Volume Slicer"))
        self.action_2.setText(_translate("GENFIRE_MainWindow", "Exit"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ProjectionCalculator_MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ProjectionCalculator(object):
    def setupUi(self, ProjectionCalculator):
        ProjectionCalculator.setObjectName("ProjectionCalculator")
        ProjectionCalculator.resize(819, 1000)
        self.centralwidget = QtWidgets.QWidget(ProjectionCalculator)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btn_clearModel = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_clearModel.sizePolicy().hasHeightForWidth())
        self.btn_clearModel.setSizePolicy(sizePolicy)
        self.btn_clearModel.setObjectName("btn_clearModel")
        self.horizontalLayout_4.addWidget(self.btn_clearModel)
        self.btn_go = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_go.sizePolicy().hasHeightForWidth())
        self.btn_go.setSizePolicy(sizePolicy)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(179, 179, 179))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Button, brush)
        self.btn_go.setPalette(palette)
        self.btn_go.setObjectName("btn_go")
        self.horizontalLayout_4.addWidget(self.btn_go)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 2, 0, 1, 4)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalSlider_theta = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_theta.setMaximum(3600)
        self.verticalSlider_theta.setSingleStep(10)
        self.verticalSlider_theta.setPageStep(100)
        self.verticalSlider_theta.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_theta.setObjectName("verticalSlider_theta")
        self.horizontalLayout_6.addWidget(self.verticalSlider_theta)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 2, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalSlider_phi = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_phi.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.verticalSlider_phi.setMaximum(3600)
        self.verticalSlider_phi.setSingleStep(1)
        self.verticalSlider_phi.setPageStep(100)
        self.verticalSlider_phi.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_phi.setTickPosition(QtWidgets.QSlider.NoTicks)
        self.verticalSlider_phi.setTickInterval(1)
        self.verticalSlider_phi.setObjectName("verticalSlider_phi")
        self.horizontalLayout_5.addWidget(self.verticalSlider_phi)
        self.gridLayout.addLayout(self.horizontalLayout_5, 0, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalSlider_psi = QtWidgets.QSlider(self.centralwidget)
        self.verticalSlider_psi.setMaximum(360)
        self.verticalSlider_psi.setSingleStep(10)
        self.verticalSlider_psi.setPageStep(100)
        self.verticalSlider_psi.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider_psi.setObjectName("verticalSlider_psi")
        self.horizontalLayout_7.addWidget(self.verticalSlider_psi)
        self.gridLayout.addLayout(self.horizontalLayout_7, 0, 2, 1, 1)
        self.lineEdit_psi = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_psi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_psi.setObjectName("lineEdit_psi")
        self.gridLayout.addWidget(self.lineEdit_psi, 1, 2, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setAlignment(QtCore.Qt.AlignCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 2, 2, 1, 1)
        self.lineEdit_theta = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_theta.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_theta.setObjectName("lineEdit_theta")
        self.gridLayout.addWidget(self.lineEdit_theta, 1, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.centralwidget)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 2, 1, 1, 1)
        self.lineEdit_phi = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_phi.sizePolicy().hasHeightForWidth())
        self.lineEdit_phi.setSizePolicy(sizePolicy)
        self.lineEdit_phi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_phi.setObjectName("lineEdit_phi")
        self.gridLayout.addWidget(self.lineEdit_phi, 1, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 3, 1, 1)
        self.verticalLayout_figure = QtWidgets.QVBoxLayout()
        self.verticalLayout_figure.setObjectName("verticalLayout_figure")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_figure.addLayout(self.horizontalLayout_9)
        self.gridLayout_2.addLayout(self.verticalLayout_figure, 0, 1, 1, 2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.lineEdit_modelFile = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEdit_modelFile.setObjectName("lineEdit_modelFile")
        self.horizontalLayout.addWidget(self.lineEdit_modelFile)
        self.btn_selectModel = QtWidgets.QPushButton(self.centralwidget)
        self.btn_selectModel.setObjectName("btn_selectModel")
        self.horizontalLayout.addWidget(self.btn_selectModel)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 4)
        self.gridLayout_3.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        ProjectionCalculator.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(ProjectionCalculator)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 819, 22))
        self.menubar.setObjectName("menubar")
        ProjectionCalculator.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(ProjectionCalculator)
        self.statusbar.setObjectName("statusbar")
        ProjectionCalculator.setStatusBar(self.statusbar)

        self.retranslateUi(ProjectionCalculator)
        QtCore.QMetaObject.connectSlotsByName(ProjectionCalculator)
        ProjectionCalculator.setTabOrder(self.lineEdit_phi, self.lineEdit_theta)
        ProjectionCalculator.setTabOrder(self.lineEdit_theta, self.lineEdit_psi)
        ProjectionCalculator.setTabOrder(self.lineEdit_psi, self.lineEdit_modelFile)
        ProjectionCalculator.setTabOrder(self.lineEdit_modelFile, self.btn_selectModel)

    def retranslateUi(self, ProjectionCalculator):
        _translate = QtCore.QCoreApplication.translate
        ProjectionCalculator.setWindowTitle(_translate("ProjectionCalculator", "Projection Calculator"))
        self.btn_clearModel.setText(_translate("ProjectionCalculator", "Clear Model"))
        self.btn_go.setText(_translate("ProjectionCalculator", "Calculate Projection Series From Model"))
        self.label_6.setText(_translate("ProjectionCalculator", "Phi"))
        self.label_8.setText(_translate("ProjectionCalculator", "Psi"))
        self.label_7.setText(_translate("ProjectionCalculator", "Theta"))
        self.label_2.setText(_translate("ProjectionCalculator", "Model"))
        self.btn_selectModel.setText(_translate("ProjectionCalculator", "Browse"))




from setuptools import find_packages,setup, Extension
from subprocess import call
import sys

setup(
	name 				= "genfire",
	packages		    = find_packages(),
	version 			= "1.1.9",
	description 		= "GENeralized Fourier Iterative REconstruction",
	author 				= "Alan (AJ) Pryor, Jr.",
    author_email 		= "apryor6@gmail.com",
    install_requires	= ['Pillow>=4.1.1','numpy>=1.12.1','matplotlib>=2.0.2','scipy>=0.19.0', 'pyparsing>=2.2.0','six>=1.10.0', 'PyQt5>=5.5.0'],
	scripts				= ['bin/genfire']
)





"""
* genfire.main *

The primary control module for running GENFIRE reconstructions.


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""

from __future__ import division
import numpy as np
import genfire
import sys
import os
from genfire.reconstruct import ReconstructionParameters


def main_InteractivelySetParameters():
    #######################################################################################################################
    ############################################### User Parameters  ######################################################
    #######################################################################################################################

    # GENFIRE's reconstruction parameters can be edited here by the user and run interactively, any inputs provided
    # by either the command line or the GUI will override these momentarily

    filename_projections = '../data/projections.mat'  #filename of projections, which should be size NxNxN_projections where N_projections is the number of projections
    filename_angles = '../data/angles.mat'  #angles can be either a 1xN_projections array containing a single tilt series, or 3xN_projections array containing 3 Euler angles for each projections in the form [phi;theta;psi]
    filename_support = '../data/support60.mat'  #NxNxN binary array specifying a region of 1's in which the reconstruction can exist
    filename_initialObject = None  #initial object to use in reconstruction; set to None to provide no initial guess
    filename_results = 'GENFIRE_rec.mrc'  #filename to save results
    resolutionExtensionSuppressionState = 2 # 1) Turn on resolution extension/suppression, 2) No resolution extension/suppression, 3) Just resolution extension

    numIterations = 100  #number of iterations to run in iterative reconstruction
    oversamplingRatio = 3  #input projections will be padded internally to match this oversampling ratio. If you prepad your projections, set this to 1
    interpolationCutoffDistance = 0.7  #radius of spherical interpolation kernel (in pixels) within which to include measured datapoints
    doYouWantToDisplayFigure = True
    displayFigure = genfire.reconstruct.DisplayFigure()
    displayFigure.DisplayFigureON = doYouWantToDisplayFigure
    calculateRFree = True
    if filename_support is None:
        useDefaultSupport = True
    else:
        useDefaultSupport = False

    reconstruction_parameters                                      = ReconstructionParameters()
    reconstruction_parameters.projectionFilename                   = filename_projections
    reconstruction_parameters.angleFilename                        = filename_angles
    reconstruction_parameters.supportFilename                      = filename_support
    reconstruction_parameters.interpolationCutoffDistance          = interpolationCutoffDistance
    reconstruction_parameters.numIterations                        = numIterations
    reconstruction_parameters.oversamplingRatio                    = oversamplingRatio
    reconstruction_parameters.displayFigure                        = displayFigure
    reconstruction_parameters.calculateRfree                       = calculateRFree
    reconstruction_parameters.resolutionExtensionSuppressionState  = resolutionExtensionSuppressionState
    reconstruction_parameters.useDefaultSupport                    = useDefaultSupport
    if os.path.isfile(filename_results): # If a valid initial object was provided, use it
        reconstruction_parameters.initialObjectFilename           = filename_results

    main(reconstruction_parameters)

def main(reconstruction_parameters):
    import genfire.fileio

    filename_projections                    = reconstruction_parameters.projectionFilename
    filename_angles                         = reconstruction_parameters.angleFilename
    filename_support                        = reconstruction_parameters.supportFilename
    filename_results                        = reconstruction_parameters.resultsFilename
    numIterations                           = reconstruction_parameters.numIterations
    oversamplingRatio                       = reconstruction_parameters.oversamplingRatio
    interpolationCutoffDistance             = reconstruction_parameters.interpolationCutoffDistance
    displayFigure                           = reconstruction_parameters.displayFigure
    resolutionExtensionSuppressionState     = reconstruction_parameters.resolutionExtensionSuppressionState
    calculateRFree                          = reconstruction_parameters.calculateRfree
    useDefaultSupport                       = reconstruction_parameters.useDefaultSupport
    use_positivity                          = reconstruction_parameters.constraint_positivity
    use_support                             = reconstruction_parameters.constraint_support
    gridding_method                         = reconstruction_parameters.griddingMethod
    enforceResolutionCircle                 = reconstruction_parameters.enforceResolutionCircle
    permitMultipleGridding                  = reconstruction_parameters.permitMultipleGridding

    if reconstruction_parameters.isInitialObjectDefined:
        filename_initialObject          = reconstruction_parameters.initialObjectFilename
    else:
        filename_initialObject               = None

    ### begin reconstruction ###
    projections = genfire.fileio.loadProjections(filename_projections) # load projections into a 3D numpy array

    # get dimensions of array and determine the array size after padding
    dims = np.shape(projections)
    paddedDim = dims[0] * oversamplingRatio
    padding = int((paddedDim-dims[0])/2)

    # load the support, or generate one if none was provided
    if useDefaultSupport or filename_support == "":
        support = np.ones((dims[0],dims[0],dims[0]),dtype=float)
    else:
        support = (genfire.fileio.readVolume(filename_support) != 0).astype(bool)

    displayFigure.reconstructionDisplayWindowSize = np.shape(support) # this is used to show the central region of reconstruction

    # now zero-pad to match the oversampling ratio
    support = np.pad(support,((padding,padding),(padding,padding),(padding,padding)),'constant')
    projections = np.pad(projections,((padding,padding),(padding,padding),(0,0)),'constant')

    #load initial object, or initialize it to zeros if none was given
    if filename_initialObject is not None and os.path.isfile(filename_initialObject):
        initialObject = genfire.fileio.readVolume(filename_initialObject)
        initialObject = np.pad(initialObject,((padding,padding),(padding,padding),(padding,padding)),'constant')
    else:
        initialObject = np.zeros_like(support)

    # load angles and check that the dimensions match the number of provided projections and that they
    # are either 1 x num_projections or 3 x num_projections
    angles = genfire.fileio.loadAngles(filename_angles)
    if np.shape(angles)[1] > 3:
        raise ValueError("Error! Dimension of angles incorrect.")
    if np.shape(angles)[1] == 1:
        tmp = np.zeros([np.shape(angles)[1], 3])
        tmp[1, :] = angles
        angles = tmp
        del tmp

    # grid the projections
    if gridding_method == "DFT":
        measuredK = genfire.reconstruct.fillInFourierGrid_DFT(projections, angles, interpolationCutoffDistance, enforceResolutionCircle)
    else:
        measuredK = genfire.reconstruct.fillInFourierGrid(projections, angles, interpolationCutoffDistance, enforceResolutionCircle, permitMultipleGridding)

    # the grid is assembled with the origin at the geometric center of the array, but for efficiency in the
    # iterative algorithm the origin is shifted to array position [0,0,0] to avoid unnecessary fftshift calls
    measuredK = np.fft.ifftshift(measuredK)

    # create a map of the spatial frequency to be used to control resolution extension/suppression behavior
    K_indices = genfire.utility.generateKspaceIndices(support)
    K_indices = np.fft.fftshift(K_indices)
    resolutionIndicators = np.zeros_like(K_indices)
    resolutionIndicators[measuredK != 0] = 1-K_indices[measuredK != 0]

    # if calculating Rfree, setup some infrastructure
    if calculateRFree:
        R_freeInd_complexX = []
        R_freeInd_complexY = []
        R_freeInd_complexZ = []
        R_freeVals_complex = []
        shell_thickness_pixels = 1 # pixel thickness of an individual shell of Rfree points
        numberOfBins = int(round(dims[0]/2/shell_thickness_pixels)) # number of frequency bins. Rfree will be tracked within each shell separately
        percentValuesForRfree = 0.05 # percentage of measured points to withhold
        spatialFrequencyForRfree = np.linspace(0,1,numberOfBins+1)
        K_indicesSmall =(K_indices)[:, :, 0:(np.shape(measuredK)[-1]//2+1)]

        for shellNum in range(0,numberOfBins):
            # collect relevant points
            measuredPointInd_complex = np.where((measuredK[:, :, 0:(np.shape(measuredK)[-1]//2+1)] != 0) & (K_indicesSmall>=(spatialFrequencyForRfree[shellNum])) & (K_indicesSmall<=(spatialFrequencyForRfree[shellNum+1])))

            # randomly shuffle
            shuffledPoints = np.random.permutation(np.shape(measuredPointInd_complex)[1])
            measuredPointInd_complex = (measuredPointInd_complex[0][shuffledPoints], measuredPointInd_complex[1][shuffledPoints], measuredPointInd_complex[2][shuffledPoints])

            # determine how many values to take
            cutoffInd_complex = np.floor(np.shape(measuredPointInd_complex)[1] * percentValuesForRfree).astype(int)

            # collect the Rfree values and coordinates
            R_freeInd_complexX.append(measuredPointInd_complex[0][:cutoffInd_complex])
            R_freeInd_complexY.append(measuredPointInd_complex[1][:cutoffInd_complex])
            R_freeInd_complexZ.append(measuredPointInd_complex[2][:cutoffInd_complex])
            R_freeVals_complex.append(measuredK[R_freeInd_complexX[shellNum], R_freeInd_complexY[shellNum], R_freeInd_complexZ[shellNum] ])

            # delete the points from the measured data
            measuredK[R_freeInd_complexX[shellNum], R_freeInd_complexY[shellNum], R_freeInd_complexZ[shellNum]] = 0

        # create tuple of coordinates
        R_freeInd_complex = [[R_freeInd_complexX], [R_freeInd_complexY], [R_freeInd_complexZ]]
        del R_freeInd_complexX
        del R_freeInd_complexY
        del R_freeInd_complexZ
    else:
        R_freeInd_complex = []
        R_freeVals_complex = []

    if resolutionExtensionSuppressionState==1: # resolution extension/suppression
        constraintEnforcementDelayIndicators = np.array(np.concatenate((np.arange(0.95, -.25, -0.15), np.arange(-0.15, .95, .1)), axis=0))
    elif resolutionExtensionSuppressionState==2:# no resolution extension or suppression
        constraintEnforcementDelayIndicators = np.array([-999, -999, -999, -999])
    elif resolutionExtensionSuppressionState==3:# resolution extension only
        constraintEnforcementDelayIndicators = np.concatenate((np.arange(0.95, -.15, -0.15),[-0.15, -0.15, -0.15]))
    else:
        print("Warning! Input resolutionExtensionSuppressionState does not match an available option. Deactivating dynamic constraint enforcement and continuing.\n")
        constraintEnforcementDelayIndicators = np.array([-999, -999, -999, -999])

    reconstructionOutputs = genfire.reconstruct.reconstruct(numIterations, np.fft.fftshift(initialObject), np.fft.fftshift(support), (measuredK)[:, :, 0:(np.shape(measuredK)[-1] // 2 + 1)], (resolutionIndicators)[:, :, 0:(np.shape(measuredK)[-1] // 2 + 1)], constraintEnforcementDelayIndicators, R_freeInd_complex, R_freeVals_complex, displayFigure, use_positivity, use_support)

    # reclaim original array size. ncBig is center of oversampled array, and n2 is the half-width of original array
    ncBig = paddedDim//2
    n2 = dims[0]//2
    reconstructionOutputs['reconstruction'] = reconstructionOutputs['reconstruction'][ncBig-n2:ncBig+n2,ncBig-n2:ncBig+n2,ncBig-n2:ncBig+n2]
    genfire.fileio.saveResults(reconstructionOutputs, filename_results)

if __name__ == "__main__" and len(sys.argv) == 1:
    print ("starting with user parameters")
    main_InteractivelySetParameters()
elif __name__ == "__main__":
    if len(sys.argv) > 1: # Parse inputs provided either from the GUI or from the command line
        inputArgumentOptions = {"-p" :  "filename_projections",
                                "-a" :  "filename_angles",
                                "-s" :  "filename_support",
                                "-o" :  "filename_results",
                                "-i" :  "filename_initialObject",
                                "-r" :  "resolutionExtensionSuppressionState",
                                "-it":  "numIterations",
                                "-or":  "oversamplingRatio",
                                "-t" :  "interpolationCutoffDistance",
                                "-d" :  "displayFigure",
                                "-rf":  "calculateRFree"
                                }
        print (sys.argv[:])
        if len(sys.argv)%2==0:
            raise Exception("Number of input options and input arguments does not match!")
        for argumentNum in range(1,len(sys.argv),2):
            print (inputArgumentOptions[sys.argv[argumentNum]])
            print  (sys.argv[argumentNum+1])
            print (inputArgumentOptions[sys.argv[argumentNum]] + "=" + sys.argv[argumentNum+1])


            exec(inputArgumentOptions[sys.argv[argumentNum]] + "= '" + sys.argv[argumentNum+1] +"'")
            print("Setting argument %s from option %s equal to GENFIRE parameter %s " % (sys.argv[argumentNum+1],sys.argv[argumentNum], inputArgumentOptions[sys.argv[argumentNum]] ))

        numIterations = int(numIterations)
        # displayFigure = bool(displayFigure)
        doYouWantToDisplayFigure = bool(displayFigure)
        displayFigure = genfire.reconstruct.DisplayFigure()
        displayFigure.DisplayFigureON = doYouWantToDisplayFigure
        oversamplingRatio = float(oversamplingRatio)
        resolutionExtensionSuppressionState = int(resolutionExtensionSuppressionState)
        calculateRFree = bool(calculateRFree)
        try:
            main(filename_projections,
                 filename_angles,
                 filename_support,
                 filename_results,
                 numIterations,
                 oversamplingRatio,
                 interpolationCutoffDistance,
                 resolutionExtensionSuppressionState,
                 displayFigure,
                 calculateRFree,
                 filename_initialObject)
        except (NameError, IOError):
             main(filename_projections,
                  filename_angles,
                  filename_support,
                  filename_results,
                  numIterations,
                  oversamplingRatio,
                  interpolationCutoffDistance,
                  resolutionExtensionSuppressionState,
                  displayFigure,
                  calculateRFree)



from . import utility
from . import VolumeSlicer
from . import launch
from . import ProjectionCalculator
from . import GENFIRE_MainWindow
from . import ProjectionCalculator_MainWindow
from . import CalculateProjectionSeries_Dialog
from . import VolumeSlicer_MainWindow



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'VolumeSlicer_MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VolumeSlicer(object):
    def setupUi(self, VolumeSlicer):
        VolumeSlicer.setObjectName("VolumeSlicer")
        VolumeSlicer.resize(1800, 800)
        self.centralwidget = QtWidgets.QWidget(VolumeSlicer)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.hz_lyt_figures = QtWidgets.QHBoxLayout()
        self.hz_lyt_figures.setObjectName("hz_lyt_figures")
        self.vt_lyt_fig1 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig1.setObjectName("vt_lyt_fig1")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig1)
        self.vt_lyt_fig2 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig2.setObjectName("vt_lyt_fig2")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig2)
        self.vt_lyt_fig3 = QtWidgets.QVBoxLayout()
        self.vt_lyt_fig3.setObjectName("vt_lyt_fig3")
        self.hz_lyt_figures.addLayout(self.vt_lyt_fig3)
        self.verticalLayout_2.addLayout(self.hz_lyt_figures)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.hz_lyt_sliders = QtWidgets.QHBoxLayout()
        self.hz_lyt_sliders.setObjectName("hz_lyt_sliders")
        self.sldr_fig1 = QtWidgets.QVBoxLayout()
        self.sldr_fig1.setObjectName("sldr_fig1")
        self.scrlbr_fig1 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig1.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig1.setInvertedAppearance(False)
        self.scrlbr_fig1.setObjectName("scrlbr_fig1")
        self.sldr_fig1.addWidget(self.scrlbr_fig1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.horizontalLayout.addLayout(self.horizontalLayout_6)
        self.lineEdit_scrlbr1 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr1.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr1.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr1.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr1.setObjectName("lineEdit_scrlbr1")
        self.horizontalLayout.addWidget(self.lineEdit_scrlbr1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.sldr_fig1.addLayout(self.horizontalLayout)
        self.hz_lyt_sliders.addLayout(self.sldr_fig1)
        self.sldr_fig2 = QtWidgets.QVBoxLayout()
        self.sldr_fig2.setObjectName("sldr_fig2")
        self.scrlbr_fig2 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig2.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig2.setObjectName("scrlbr_fig2")
        self.sldr_fig2.addWidget(self.scrlbr_fig2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem2)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_8.addWidget(self.label_2)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_8)
        self.lineEdit_scrlbr2 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr2.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr2.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr2.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr2.setObjectName("lineEdit_scrlbr2")
        self.horizontalLayout_2.addWidget(self.lineEdit_scrlbr2)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.sldr_fig2.addLayout(self.horizontalLayout_2)
        self.hz_lyt_sliders.addLayout(self.sldr_fig2)
        self.sldr_fig3 = QtWidgets.QVBoxLayout()
        self.sldr_fig3.setObjectName("sldr_fig3")
        self.scrlbr_fig3 = QtWidgets.QScrollBar(self.centralwidget)
        self.scrlbr_fig3.setOrientation(QtCore.Qt.Horizontal)
        self.scrlbr_fig3.setObjectName("scrlbr_fig3")
        self.sldr_fig3.addWidget(self.scrlbr_fig3)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem4)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_9.addWidget(self.label_3)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_9)
        self.lineEdit_scrlbr3 = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_scrlbr3.sizePolicy().hasHeightForWidth())
        self.lineEdit_scrlbr3.setSizePolicy(sizePolicy)
        self.lineEdit_scrlbr3.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_scrlbr3.setObjectName("lineEdit_scrlbr3")
        self.horizontalLayout_3.addWidget(self.lineEdit_scrlbr3)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem5)
        self.sldr_fig3.addLayout(self.horizontalLayout_3)
        self.hz_lyt_sliders.addLayout(self.sldr_fig3)
        self.verticalLayout.addLayout(self.hz_lyt_sliders)
        self.hz_lyt_btns = QtWidgets.QHBoxLayout()
        self.hz_lyt_btns.setObjectName("hz_lyt_btns")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.checkBox_lockcmap = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_lockcmap.setObjectName("checkBox_lockcmap")
        self.horizontalLayout_5.addWidget(self.checkBox_lockcmap)
        self.hz_lyt_btns.addLayout(self.horizontalLayout_5)
        self.verticalLayout.addLayout(self.hz_lyt_btns)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        VolumeSlicer.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(VolumeSlicer)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1800, 22))
        self.menubar.setObjectName("menubar")
        VolumeSlicer.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(VolumeSlicer)
        self.statusbar.setObjectName("statusbar")
        VolumeSlicer.setStatusBar(self.statusbar)

        self.retranslateUi(VolumeSlicer)
        QtCore.QMetaObject.connectSlotsByName(VolumeSlicer)

    def retranslateUi(self, VolumeSlicer):
        _translate = QtCore.QCoreApplication.translate
        VolumeSlicer.setWindowTitle(_translate("VolumeSlicer", "Volume Slicer"))
        self.label.setText(_translate("VolumeSlicer", "X = "))
        self.label_2.setText(_translate("VolumeSlicer", "Y = "))
        self.label_3.setText(_translate("VolumeSlicer", "Z = "))
        self.checkBox_lockcmap.setText(_translate("VolumeSlicer", "Lock Colormap"))

